// simple PWM example using PD12,13,14,15 
// TIM4_CH1,2,3,4

#include "stm32f303xx.h"
#define TIM4_INPUT_FREQ 72000000
#define PWM_FREQ 20000
void delay(int);

void delay(int dly)
{
  while( dly--);
}

void configPins()
{
	// Power up PORTD
	RCC_AHBENR |= BIT20;
	// Turn on alternate function for BITS 12,13,14,15
	GPIOD_MODER |= BIT31+BIT29+BIT27+BIT25;
	GPIOD_MODER &= ~(BIT30+BIT28+BIT26+BIT24);
	// Select AF2 alternate function (Timer 4 output)
	GPIOD_AFRH &= 0x0000ffff;
	GPIOD_AFRH |= BIT29+BIT25+BIT21+BIT17;
	
}
void initTimer4()
{
	// power up Timer 4
	RCC_APB1ENR |= BIT2;
	TIM4_ARR = TIM4_INPUT_FREQ/PWM_FREQ;
	TIM4_CCR1 = 0;
	TIM4_CCR2 = 0;
	TIM4_CCR3 = 0;
	TIM4_CCR4 = 0;
	
	// Channels 1 and 2
	// pwm mode 1: OCxM = 110 in TIM4_CCMRx = pwm mode 1
	// set OCxPE bit in TIM4_CCMRx
	TIM4_CCMR1 = (BIT14 + BIT13 +  BIT11 + BIT10 \
				  + BIT6 + BIT5 + BIT3 + BIT2); 
	// Channels 3 and 4
	TIM4_CCMR2 = (BIT14 + BIT13  + BIT11 + BIT10 \
				  + BIT6 + BIT5  + BIT3 + BIT2); 
		
	//  OCx output is enabled by the CCxE bit in
	//  the TIM4_CCER register. 
	TIM4_CCER = BIT12 + BIT8 + BIT4 + BIT0;
	
	// set ARPE in CCR1 to enable reload 
	TIM4_CR1 |= BIT7;
	
	// set UG bit in TIM4_EGR
	TIM4_EGR |= BIT0;
	
	// Enable the counter
	TIM4_CR1 |= BIT0;
}
void setDuty(int Channel, int Duty)
{
	// Duty is expressed as a percentage.
	switch (Channel)
	{
		case 1: {
			TIM4_CCR1 = (Duty*TIM4_ARR)/100;
			break;
		}
		case 2: {
			TIM4_CCR2 = (Duty*TIM4_ARR)/100;
			break;
		}
		case 3: {
			TIM4_CCR3 = (Duty*TIM4_ARR)/100;
			break;
		}
		case 4: {
			TIM4_CCR4 = (Duty*TIM4_ARR)/100;
			break;
		}		
	}
}
void initClock()
{
	// Assumption: Using HSE clock (8MHz)
	// This clock is provided by the STLink debug interface
	// PLL is driven by HSE = 8MHz
	// Multiply x 9 = 72MHz
	
	// Must set flash latency (2 wait states) to allow
	// for higher speed clock	
	FLASH_ACR |= 2; // add two wait states
	
	RCC_CR &= ~BIT24; // turn off PLL
	delay(10);	
	RCC_CFGR |= ((0x07) << 18);	// PLL multiply = 9
	RCC_CFGR |= BIT16;  // select HSE as PLL input
	RCC_CFGR |= BIT10;  // APB1 Clock = HCLK/2 (36MHz)
	RCC_CR |= BIT18 + BIT16; // Turn on HSE and set HSEBYP
	delay(10);
	RCC_CR |= BIT24; // turn on PLL
	delay(10);
	RCC_CFGR |= BIT1;  // switch to PLL as clock
}

int main()
{	
	int debug;
	initClock();	
	configPins();
	initTimer4();
	setDuty(1,20);
	setDuty(2,40);
	setDuty(3,60);
	setDuty(4,80);
	while(1)
	{					
		debug = RCC_CFGR;
		delay(100000);
	} 
	return 0;
}








